/* $Id: ast_datatypecasestatement.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_DATATYPECASESTATEMENT_H
#define WTPARSER_AST_DATATYPECASESTATEMENT_H

#include "../astnode.h"
#include "ast_switchcase.h"
#include <deque>

namespace wtast
{

class DataTypeCons;
class Compound;
class VariableDeclaration;

class DataTypeCaseStatement : public SwitchCase
{
	typedef std::deque<VariableDeclaration*> vardecls_t;

	DataTypeCons* mCons;
	vardecls_t    mVarDecls;
	Compound*     mBody;

public:
	DataTypeCaseStatement(DataTypeCons* cons, Compound* body)
		: SwitchCase()
		, mCons(cons)
		, mBody(body)
	{
		// ok
	}

	~DataTypeCaseStatement();

	void Accept(AstVisitor* vis);

	DataTypeCaseStatement* Copy();

	Kind GetKind() const
	{
		return DATATYPECASESTATEMENT;
	}

	template<typename Iter>
	void AddVariables(Iter start, Iter end)
	{
		for(;start!=end;++start)
		{
			AddVariable(*start);
		}
	}

	void AddVariable(VariableDeclaration* vardecl)
	{
		mVarDecls.push_back(vardecl);
	}

	std::size_t GetNumberOfVariables() const
	{
		return mVarDecls.size();
	}

	VariableDeclaration* GetVariable(std::size_t idx)
	{
		if(idx<mVarDecls.size())
		{
			return mVarDecls[idx];
		}
		else
		{
			return 0;
		}
	}

	DataTypeCons* GetCons()
	{
		return mCons;
	}

	Compound* GetBody()
	{
		return mBody;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_DATATYPECASESTATEMENT_H
