/* $Id: ast_simultaneoustype.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_simultaneoustype.h"
#include "ast_type.h"
#include "../astvisitor.h"
#include <sstream>

wtast::SimultaneousType::~SimultaneousType()
{
}

void wtast::SimultaneousType::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		for(types_t::iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

wtast::SimultaneousType* wtast::SimultaneousType::Copy()
{
	SimultaneousType* st = new SimultaneousType();

	for(types_t::iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		st->AddType(*it);
	}

	return st;
}

std::string wtast::SimultaneousType::GetName() const
{
	std::ostringstream oss;
	bool comma = false;

	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;comma=true,++it)
	{
		if(comma)
		{
			oss << ", ";
		}

		oss << (*it)->GetName();
	}

	return oss.str();
}

bool wtast::SimultaneousType::HasObject() const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if((*it)->IsObject())
		{
			return true;
		}
	}

	return false;
}

bool wtast::SimultaneousType::HasFunctionType() const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if((*it)->HasFunctionType())
		{
			return true;
		}
	}

	return false;
}

bool wtast::SimultaneousType::IsCopyable() const
{
	for(types_t::const_iterator it=mTypes.begin(),end=mTypes.end();it!=end;++it)
	{
		if(!(*it)->IsCopyable())
		{
			return false;
		}
	}

	return true;
}
