#ifndef WTRUNTIME_FUNCSIG_H
#define WTRUNTIME_FUNCSIG_H

#include "config.h"
#include "types.h"
#include "byteorder.h"
#include "wtruntime.h"

struct wtr_profile;
struct wtr_program;

struct wtr_funcsig
{
	wt_u32      argc; /* number of arguments */
	wt_u32      retc; /* number of return types */
	wt_uintptr  pure; /* non-zero if the function is pure */
	const char* name; /* name of the function */
	wt_u32*     argv; /* argument type array (resolved types only) */
	wt_u32*     retv; /* return type array (resolved types only) */
};

/* Create a new function signature. Types passed to this function may have */
/* unresolved IDs (to resolve these types, the program must be passed as */
/* a resolving context. Also, the source arrays ('argv' and 'retv') may have */
/* different byte-order (in this case the 'bo' shall be set accordingly). */
struct wtr_funcsig* wtr_funcsig_new(struct wtr_profile* prof, const char* name,
	wt_u32 argc, const wt_u32* argv, wt_u32 retc, const wt_u32* retv,
	wt_bool pure, struct wtr_program* prog, wt_bo bo);

void wtr_funcsig_delete(struct wtr_profile* prof, struct wtr_funcsig* fsig);

int wtr_funcsig_cmp(const struct wtr_funcsig* lhs, const struct wtr_funcsig* rhs);
int wtr_funcsig_cmp_with_retv(const struct wtr_funcsig* lhs, const struct wtr_funcsig* rhs);

#endif /* WTRUNTIME_FUNCSIG_H */
